/*
 * 文件名: UnionService.java
 * 版    权：  Copyright Changwan Tech. Co. Ltd. All Rights Reserved.
 * 描    述: [该类的简要描述]
 * 创建人: turlet@163.com
 * 创建时间:2013-5-8
 * 
 * 修改人：
 * 修改时间:
 * 修改内容：[修改内容]
 */
package cn.ewan.pushsdk.internal;

import java.util.List;

import android.app.ActivityManager;
import android.app.ActivityManager.RunningServiceInfo;
import android.content.Context;
import android.content.Intent;
import android.util.Log;
import cn.ewan.pushsdk.client.IMqttActionListener;
import cn.ewan.pushsdk.client.IMqttToken;
import cn.ewan.pushsdk.client.MqttConnectOptions;
import cn.ewan.pushsdk.client.MqttException;
import cn.ewan.pushsdk.client.MqttMessage;
import cn.ewan.pushsdk.logic.CacheData;
import cn.ewan.pushsdk.open.EwanPushCallback;
import cn.ewan.pushsdk.open.EwanPushCode;
import cn.ewan.pushsdk.service.EwanMessageService;
import cn.ewan.pushsdk.service.MqttAndroidClient;
import cn.ewan.pushsdk.util.PushTraceCallback;

/**
 * [联运SDK的所有业务接口]<BR>
 * [功能详细描述]
 * @author turlet@163.com
 * @version [Client V100R001C03, 2013-5-8] 
 */
public class EwanPushCliendApi {
    
    private static final String TAG = EwanPushCliendApi.class.getSimpleName();
    
    private static EwanPushCliendApi instance = null;
    
    
    /** Default QOS value*/
    private static final int defaultQos = 0;
    /** Default timeout*/
    private static final int defaultTimeOut = 10;
    /** Default keep alive value*/
    private static final int defaultKeepAlive = 20;
    
    private MqttAndroidClient client;
    
    private Context ctx;
    
    private static final int MAX_LIST_SRV = 50;
    
    public EwanPushCliendApi(Context context)
	{
    	this.ctx = context;
	}

	public synchronized static EwanPushCliendApi getInstance(Context context)
    {
      if (instance == null) {
        instance = new EwanPushCliendApi(context);
      }

      return instance;
    }
     
    private MqttAndroidClient createClient(Context context, String serverURI, String clientId)
    {
	      MqttAndroidClient client = new MqttAndroidClient(context, serverURI, clientId);
	      return client;
    }
    
    public void connect(String uri, String clientId,final EwanPushCallback callback)
	{
    	this.client = createClient(ctx, uri, clientId);
    	
    	MqttConnectOptions conOpt = new MqttConnectOptions();
        conOpt.setCleanSession(false);
        conOpt.setConnectionTimeout(defaultTimeOut);
        conOpt.setKeepAliveInterval(defaultKeepAlive);
        conOpt.setMqttVersion(3);
        
//        client.registerResources(ctx);
        client.setCallback(new EwanPushCliendDataCallbackHandler(ctx));

        //set traceCallback
        client.setTraceCallback(new PushTraceCallback());
        client.setTraceEnabled(true);
    	
        try {
        	this.client.connect(conOpt, null, new IMqttActionListener()
			{
				
				@Override
				public void onSuccess(IMqttToken asyncActionToken)
				{
					if(callback != null){
						callback.onSuccess();
					}
				}
				
				@Override
				public void onFailure(IMqttToken asyncActionToken, Throwable exception)
				{
					if(callback != null){
						callback.onFail(EwanPushCode.ERROR_NETWORD_DISCONNECT, "建立连接失败=="+exception.toString());
					}
				}
			});
        	client.registerResources(ctx);
          }
          catch (MqttException e) {
            Log.e(this.getClass().getCanonicalName(),
                "EwanPushCliendApi connect fail====", e);
			if(callback != null){
				callback.onFail(EwanPushCode.ERROR_NETWORD_DISCONNECT, "建立连接失败");
			}
          }
	}
    
    public void disConnect(){
    	
    	if(this.client != null){    
    		try
    		{
    			if(this.client.isConnected()){
    				this.client.disconnect();
    			}
    		}
    		catch (MqttException e)
    		{
    			e.printStackTrace();
    		}
    		client.unregisterResources();
    		client = null;
    	}
    }
    
    
    public void subscribe(String topics,final EwanPushCallback  callback){
    	if(this.client != null){
//    		CacheData.getInstance().saveTopics(ctx, topics);
    		try
			{
				this.client.subscribe(topics, defaultQos, null, new IMqttActionListener()
				{
					
					@Override
					public void onSuccess(IMqttToken asyncActionToken)
					{
						if(callback != null){
							callback.onSuccess();
						}
					}
					
					@Override
					public void onFailure(IMqttToken asyncActionToken, Throwable exception)
					{
						if(callback != null){
							callback.onFail(EwanPushCode.ERROR_SUBSCRIBE_FAIL, "订阅失败");
						}
					}
				});
			}
			catch (MqttException e)
			{
				// TODO 自动生成的 catch 块
				e.printStackTrace();
				if(callback != null){
					callback.onFail(EwanPushCode.ERROR_SUBSCRIBE_FAIL, "订阅失败");
				}
			}
    	}
    }
    
    public void subscribe(String[] topics,final EwanPushCallback  callback){
    	if(topics == null || topics.length <= 0){
    		return;
    	}
    	
    	int[] qos = new int[topics.length];
    	if(this.client != null){
    		for(int i=0; i < topics.length; i++){
//    			CacheData.getInstance().saveTopics(ctx, topics[i]);
    			qos[i] = defaultQos;
    		}
    		
    		try
			{
				this.client.subscribe(topics, qos, null, new IMqttActionListener()
				{
					
					@Override
					public void onSuccess(IMqttToken asyncActionToken)
					{
						if(callback != null){
							callback.onSuccess();
						}
					}
					
					@Override
					public void onFailure(IMqttToken asyncActionToken, Throwable exception)
					{
						if(callback != null){
							callback.onFail(EwanPushCode.ERROR_SUBSCRIBE_FAIL, "订阅失败");
						}
					}
				});
			}
			catch (MqttException e)
			{
				e.printStackTrace();
				if(callback != null){
					callback.onFail(EwanPushCode.ERROR_SUBSCRIBE_FAIL, "订阅失败");
				}
			}
    	}
    }
    
    public void unSubscribe(String topics,final EwanPushCallback  callback){
    	if(this.client != null){
//    		CacheData.getInstance().removeTopics(ctx, topics);
    		try
			{
				this.client.unsubscribe(topics, null, new IMqttActionListener()
				{
					
					@Override
					public void onSuccess(IMqttToken asyncActionToken)
					{
						if(callback != null){
							callback.onSuccess();
						}
					}
					
					@Override
					public void onFailure(IMqttToken asyncActionToken, Throwable exception)
					{
						if(callback != null){
							callback.onFail(EwanPushCode.ERROR_UNSUBSCRIBE_FAIL, "取消订阅失败");
						}
					}
				});
			}
			catch (MqttException e)
			{
				// TODO 自动生成的 catch 块
				e.printStackTrace();
				if(callback != null){
					callback.onFail(EwanPushCode.ERROR_UNSUBSCRIBE_FAIL, "取消订阅失败");
				}
			}
    	}
    }
    
    
    public void unSubscribe(String[] topics,final EwanPushCallback  callback){
    	if(this.client != null){
//    		CacheData.getInstance().removeTopics(ctx, topics);
    		try
			{
				this.client.unsubscribe(topics, null, new IMqttActionListener()
				{
					
					@Override
					public void onSuccess(IMqttToken asyncActionToken)
					{
						if(callback != null){
							callback.onSuccess();
						}
					}
					
					@Override
					public void onFailure(IMqttToken asyncActionToken, Throwable exception)
					{
						if(callback != null){
							callback.onFail(EwanPushCode.ERROR_UNSUBSCRIBE_FAIL, "取消订阅失败");
						}
					}
				});
			}
			catch (MqttException e)
			{
				// TODO 自动生成的 catch 块
				e.printStackTrace();
				if(callback != null){
					callback.onFail(EwanPushCode.ERROR_UNSUBSCRIBE_FAIL, "取消订阅失败");
				}
			}
    	}
    }
    
    
    public void publish(String topic, String message,final EwanPushCallback  callback){
    	if(this.client != null){
    		try
			{
				this.client.publish(topic, message.getBytes(), defaultQos, true, null, new IMqttActionListener()
				{
					
					@Override
					public void onSuccess(IMqttToken asyncActionToken)
					{
						if(callback != null){
							callback.onSuccess();
						}
					}
					
					@Override
					public void onFailure(IMqttToken asyncActionToken, Throwable exception)
					{
						if(callback != null){
							callback.onFail(EwanPushCode.ERROR_SEND_MESSAGE_FAIL, "发送消息失败");
						}
					}
				});
			}
			catch (MqttException e)
			{
				// TODO 自动生成的 catch 块
				e.printStackTrace();
				if(callback != null){
					callback.onFail(EwanPushCode.ERROR_SEND_MESSAGE_FAIL, "发送消息失败");
				}
			}
    	}
    }
    
}
