/*******************************************************************************
 * Copyright (c) 1999, 2014 IBM Corp.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * and Eclipse Distribution License v1.0 which accompany this distribution. 
 *
 * The Eclipse Public License is available at 
 *    http://www.eclipse.org/legal/epl-v10.html
 * and the Eclipse Distribution License is available at 
 *   http://www.eclipse.org/org/documents/edl-v10.php.
 */
package cn.ewan.pushsdk.internal;

import java.sql.Connection;

import org.json.JSONException;
import org.json.JSONObject;

import android.content.Context;
import cn.ewan.pushsdk.client.IMqttDeliveryToken;
import cn.ewan.pushsdk.client.MqttCallback;
import cn.ewan.pushsdk.client.MqttMessage;
import cn.ewan.pushsdk.logic.CacheData;
import cn.ewan.pushsdk.open.EwanPushClient;
import cn.ewan.pushsdk.util.AES;

/**
 * Handles call backs from the MQTT Client
 *
 */
public class EwanPushCliendDataCallbackHandler implements MqttCallback {

  /** {@link Context} for the application used to format and import external strings**/
  private Context context;

  /**
   * Creates an <code>MqttCallbackHandler</code> object
   * @param context The application's context
   * @param clientHandle The handle to a {@link Connection} object
   */
  public EwanPushCliendDataCallbackHandler(Context context)
  {
    this.context = context;
  }

  /**
   * @see org.eclipse.paho.client.mqttv3.MqttCallback#connectionLost(java.lang.Throwable)
   */
  @Override
  public void connectionLost(Throwable cause) {
//	  cause.printStackTrace();
    if (cause != null) {
    	
    }
  }

  /**
   * @see org.eclipse.paho.client.mqttv3.MqttCallback#messageArrived(java.lang.String, org.eclipse.paho.client.mqttv3.MqttMessage)
   */
  @Override
  public void messageArrived(String topic, MqttMessage message) throws Exception {
		String ret = "";
		boolean flag = false;
		try {
			JSONObject requestJson = new JSONObject(new String(message.getPayload()));
			flag = requestJson.getInt("flag") == 1 ? true : false;
			if (requestJson.has("data")) {
				String encryptData = requestJson.getString("data");
				if (flag) {
					try {
						ret =  AES.Decrypt(encryptData, CacheData.getInstance().getContentKey(context));
					} catch (Exception e) {
						ret = "";
					}
				} else {
					ret = encryptData;
				}
			}
		} catch (JSONException e) {
			e.printStackTrace();
		}
		
//		WriteLog.e("messageArrived", ret);
	  if(EwanPushClient.getInstance().getDataCallback() != null){
		  EwanPushClient.getInstance().getDataCallback().onMessageArrived(context, topic, ret);
	  }
  }

  /**
   * @see org.eclipse.paho.client.mqttv3.MqttCallback#deliveryComplete(org.eclipse.paho.client.mqttv3.IMqttDeliveryToken)
   */
  @Override
  public void deliveryComplete(IMqttDeliveryToken token) {
	  if(EwanPushClient.getInstance().getDataCallback() != null){
		  EwanPushClient.getInstance().getDataCallback().onDeliveryComplete();
	  }
  }

}
