/*
 * 文件名: AsyncHttpPostTask.java
 * 版    权：  Copyright Administrator Tech. Co. Ltd. All Rights Reserved.
 * 描    述: [该类的简要描述]
 * 创建人: 杨兴朗
 * 创建时间:2012-11-7
 * 
 * 修改人：
 * 修改时间:
 * 修改内容：[修改内容]
 */
package cn.ewan.pushsdk.net;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Map;

import android.accounts.NetworkErrorException;
import android.content.Context;
import cn.ewan.pushsdk.util.ErrorCode;
import cn.ewan.pushsdk.util.StringUtil;

/**
 * 异步post请求<BR>
 * [功能详细描述]
 * @author 杨兴朗
 * @version [RCS Client V100R001C03, 2012-11-7] 
 */
public class AsyncHttpPostTask {
    
    public void request(Context context, String url,
            Map<String, String> params, String parameters, final byte[] data,
            IHttpListener listener, String enc, boolean encryption) {
        request(context,
                url,
                params,
                parameters,
                data,
                "POST",
                listener,
                null,
                enc,
                encryption);
    }
    
    /**
     * 异步网络请求<BR>
     * [功能详细描述]
     * @param url
     * @param params
     * @param parameters
     * @param data
     * @param httpMethod
     * @param listener
     * @param state
     * @param enc 编码
     * @param encryption 数据是否加密发送
     * @see com.cw.platform.net.HttpRequest.openUrl
     */
    public void request(final Context context, final String url,
            final Map<String, String> params, final String parameters,
            final byte[] data, final String httpMethod,
            final IHttpListener listener, final Object state, final String enc,
            final boolean encryption) {
        new Thread() {
            public void run() {
                try {
                    String resp = HttpRequest.openUrl(context,
                            url,
                            httpMethod,
                            params,
                            parameters,
                            data,
                            enc,
                            encryption);
                    if (StringUtil.isEmpty(resp)) {
                        listener.onException(ErrorCode.ERROR_SERVER_BUSY,
                                new NullPointerException("服务器没有返回数据."));
                    }
                    else {
                        listener.onComplete(resp, state);
                    }
                }
                catch (FileNotFoundException e) {
                    listener.onException(ErrorCode.ERROR_SERVER_BUSY, e);
                }
                catch (IOException e) {
                    listener.onException(ErrorCode.ERROR_SERVER_BUSY, e);
                }
                catch (NetworkErrorException e) {
                    listener.onException(ErrorCode.ERROR_NETWORD_DISCONNECT, e);
                }
            }
        }.start();
    }
}
