/*
 * 文件名: HttpRequest.java
 * 版    权：  Copyright Administrator Tech. Co. Ltd. All Rights Reserved.
 * 描    述: [该类的简要描述]
 * 创建人: 杨兴朗
 * 创建时间:2012-11-7
 * 
 * 修改人：
 * 修改时间:
 * 修改内容：[修改内容]
 */
package cn.ewan.pushsdk.net;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.InetSocketAddress;
import java.net.URL;
import java.net.URLEncoder;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.Map;

import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

import android.accounts.NetworkErrorException;
import android.content.Context;
import android.database.Cursor;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.Uri;
import android.text.TextUtils;
import android.util.Log;
import cn.ewan.pushsdk.logic.CacheData;
import cn.ewan.pushsdk.util.AES;
import cn.ewan.pushsdk.util.Common;
import cn.ewan.pushsdk.util.LogUtil;
import cn.ewan.pushsdk.util.StringUtil;

/**
 * 全局网络请求出口工具类<BR>
 * [功能详细描述]
 * @author 杨兴朗
 * @version [RCS Client V100R001C03, 2012-11-7] 
 */
public class HttpRequest {
    
    private static final String TAG = "HttpRequest";
    
    public static final String CTWAP = "ctwap";
    
    public static final String CMWAP = "cmwap";
    
    public static final String WAP_3G = "3gwap";
    
    public static final String UNIWAP = "uniwap";
    
    /** 网络不可用 */
    public static final int TYPE_NET_WORK_DISABLED = 0;
    
    public static final int TYPE_WIFI = 1;
    
    /** 移动联通wap10.0.0.172 */
    public static final int TYPE_CM_CU_WAP = 4;
    
    /** 电信wap 10.0.0.200 */
    public static final int TYPE_CT_WAP = 5;
    
    /** 电信,移动,联通 */
    public static final int TYPE_OTHER_NET = 6;
    
    public static Uri PREFERRED_APN_URI = Uri.parse("content://telephony/carriers/preferapn");
    
    /**
     * 获取Network具体类型
     * 
     * @param mContext
     * @return
     */
    public static int checkNetworkType(Context mContext) {
        try {
            final ConnectivityManager connectivityManager = (ConnectivityManager) mContext.getSystemService(Context.CONNECTIVITY_SERVICE);
            final NetworkInfo networkInfo = connectivityManager.getActiveNetworkInfo();
            if (networkInfo == null || !networkInfo.isAvailable()) {
                return TYPE_NET_WORK_DISABLED;
            }
            else {
                int netType = networkInfo.getType();
                if (netType == ConnectivityManager.TYPE_WIFI) {
                    return TYPE_WIFI;
                }
                else if (netType == ConnectivityManager.TYPE_MOBILE) {
                    // 电信wap
                    final Cursor c = mContext.getContentResolver()
                            .query(PREFERRED_APN_URI, null, null, null, null);
                    try {
                        if (c != null) {
                            c.moveToFirst();
                            final String user = c.getString(c.getColumnIndex("user"));
                            if (!TextUtils.isEmpty(user)) {
                                if (user.startsWith(CTWAP)) {
                                    return TYPE_CT_WAP;
                                }
                            }
                        }
                        String netMode = networkInfo.getExtraInfo();
                        if (netMode != null) {
                            netMode = netMode.toLowerCase();
                            if (netMode.equals(CMWAP) || netMode.equals(WAP_3G)
                                    || netMode.equals(UNIWAP)) {
                                return TYPE_CM_CU_WAP;
                            }
                        }
                    }
                    catch (Exception e) {
                        e.printStackTrace();
                    }
                    finally {
                        if (c != null) {
                            c.close();
                        }
                    }
                }
            }
        }
        catch (Exception ex) {
            return TYPE_OTHER_NET;
        }
        return TYPE_OTHER_NET;
    }
    
    /**
     * 打开网络连接<BR>
     * [功能详细描述]
     * @param [context] 上下文 
     * @param [url] 地址
     * @param [method] 请求方法GET/POST
     * @param [params] 参数以Map方式传进来
     * @param [parameter] 参数组装好的参数
     * @param [outData] 以post方式直接写出去的数据流
     * @param [enc] 编码方式GBK/UTF-8
     * @param [encryption] 发送参数是否加密
     * @return
     * @throws IOException
     * @throws NetworkErrorException 
     */
    public static String openUrl(final Context context, String url, String method,
            Map<String, String> params, String parameter, final byte[] outData,
            String enc, boolean encryption) throws IOException,
            NetworkErrorException {
        String response = "";
        int type = checkNetworkType(context);
        if (TYPE_NET_WORK_DISABLED == type) {
            throw new NetworkErrorException("network is disabled");
        }
        if (method.equals("GET")) {
            if (null != params) {
                url = url + encodeUrl(context, params, enc, encryption);
            }
            else if (!StringUtil.isEmpty(parameter)) {
                url = url + "?" + parameter;
            }
        }
        HttpURLConnection conn = null;
        if (url.startsWith("https")) {
            conn = (HttpsURLConnection) new URL(url).openConnection();
            try {
                TrustManager easyTrustManager = new X509TrustManager() {
                    public void checkClientTrusted(X509Certificate[] chain,
                            String authType) throws CertificateException {
                    }
                    
                    public void checkServerTrusted(X509Certificate[] chain,
                            String authType) throws CertificateException {
                    }
                    
                    public X509Certificate[] getAcceptedIssuers() {
                        return null;
                    }
                };
                SSLContext sslcontext = SSLContext.getInstance("TLS");
                sslcontext.init(null,
                        new TrustManager[] { easyTrustManager },
                        null);
                ((HttpsURLConnection) conn).setSSLSocketFactory(sslcontext.getSocketFactory());
            }
            catch (Exception e) {
            }
        }
        else {
            if (null != params && null != outData) {
                url = url + "?" + encodeUrl(context, params, enc, encryption);
                LogUtil.i(TAG, "图片上传url==" + url);
            }
            if (type == TYPE_CM_CU_WAP || type == TYPE_CT_WAP) {//判断是wap
                try {
                    java.net.Proxy p = new java.net.Proxy(
                            java.net.Proxy.Type.HTTP, new InetSocketAddress(
                                    android.net.Proxy.getDefaultHost(),
                                    android.net.Proxy.getDefaultPort()));
                    conn = (HttpURLConnection) new URL(url).openConnection(p);
                }
                catch (Exception e) {
                    conn = (HttpURLConnection) new URL(url).openConnection();
                }
            }
            else {//正常连接
                conn = (HttpURLConnection) new URL(url).openConnection();
            }
        }
        LogUtil.i(TAG, "请求地址：" + url);
        conn.setRequestProperty("User-Agent", System.getProperties()
                .getProperty("http.agent") + " CWAndroidSDK");
        if (!"GET".equals(method)) {
            byte[] data = new byte[1];
            if (null != params) {
                data = encodeUrl(context, params, enc, encryption).getBytes();
            }
            if (!StringUtil.isEmpty(parameter)) {
                data = parameter.getBytes(enc);
            }
            if (null != outData) {
                data = outData;
            }
            
            conn.setDoInput(true);
            conn.setDoOutput(true);//发送POST请求必须设置允许输出
            conn.setUseCaches(false);//不使用Cache
            conn.setConnectTimeout(Common.connect_timeout);
            conn.setReadTimeout(Common.read_timeout);
            conn.setRequestMethod("POST");
            conn.setRequestProperty("Connection", "Keep-Alive");//维持长连接
            conn.setRequestProperty("Charset", enc);
            conn.setRequestProperty("Content-Length",
                    String.valueOf(data.length));
            if (Common.ENC_GBK.equalsIgnoreCase(enc)) {
                conn.setRequestProperty("Content-Type",
                        "application/x-www-form-urlencoded;charset=GBK");
            }
            else {
                conn.setRequestProperty("Content-Type",
                        "application/x-www-form-urlencoded");
            }
            OutputStream os = new BufferedOutputStream(conn.getOutputStream());
            os.write(data);
            os.flush();
            
        }
        try {
            response = read(context, conn.getInputStream(), encryption, enc);
        }
        catch (FileNotFoundException e) {
            response = read(context, conn.getErrorStream(), encryption, enc);
        }
        return response;
    }
    
    /**
     * [一句话功能简述]<BR>
     * [功能详细描述]
     * @param params 组装成参数
     * @param enc 编码
     * @return
     * @throws UnsupportedEncodingException
     */
    private static String encodeUrl(Context context, Map<String, String> params, String enc,
            boolean encryption) throws UnsupportedEncodingException {
        if (params == null) {
            return "";
        }
        StringBuilder sb = new StringBuilder();
        String value = "";
        for (Map.Entry<String, String> entry : params.entrySet()) {
            sb.append(entry.getKey());
            sb.append("=");
            value = entry.getValue();
            if (StringUtil.isEmpty(value)) {
                sb.append("");
            }
            else {
                sb.append(URLEncoder.encode(entry.getValue(), enc));
            }
            sb.append("&");
        }
        if (sb.length() > 0) {
            sb.deleteCharAt(sb.length() - 1);
        }
        String temp = sb.toString();
        LogUtil.i(TAG, "未加密参数：" + temp);
/*        if(Common.isInitProtocol()){
    		try
			{
				temp = AES.Encrypt(temp, Common.SIGN_KEY.substring(0, 16));
				LogUtil.i(TAG, "数据加密：" + temp);
			}
			catch (Exception e)
			{
				Log.e(TAG, "数据加密失败:" + e.getMessage());
			}	    
        }else{
        	
        	if (encryption
        			&& !StringUtil.isEmpty(CacheData.getInstance().getProtocolKey(context))) {
        		try {
        			temp = AES.Encrypt(temp, CacheData.getInstance().getProtocolKey(context));	                
        			LogUtil.i(TAG, "数据加密：" + temp);
        		}
        		catch (Exception e) {
        			Log.e(TAG, "数据加密失败:" + e.getMessage());
        		}
        	}
        }*/
    	if (encryption
    			&& !StringUtil.isEmpty(CacheData.getInstance().getProtocolKey(context))) {
    		try {
    			temp = AES.Encrypt(temp, CacheData.getInstance().getProtocolKey(context).substring(0, 16));	
    			LogUtil.i(TAG, "数据加密：" + temp);
    		}
    		catch (Exception e) {
    			Log.e(TAG, "数据加密失败:" + e.getMessage());
    		}
    	}
        return temp;
    }
    
    /**
     * 输出流转字符串<BR>
     * [功能详细描述]
     * @param in
     * @param encryption 是否需要加密
     * @return
     * @throws IOException
     */
    private static String read(Context context, InputStream in, boolean encryption, String enc)
            throws IOException {
        StringBuilder sb = new StringBuilder();
        BufferedReader r = new BufferedReader(new InputStreamReader(in, enc),
                1000);
        if (Common.ENC_GBK.equals(enc)) {
            for (String line = r.readLine(); line != null; line = r.readLine()) {
                sb.append(line).append("\n");
            }
        }
        else {
            for (String line = r.readLine(); line != null; line = r.readLine()) {
                sb.append(line);
            }
        }
        in.close();
        String ret = "";
        if (encryption) {
            try {
                LogUtil.i(TAG, "未解密数据=" + sb.toString());
            	ret = AES.Decrypt(sb.toString(), CacheData.getInstance().getProtocolKey(context).substring(0, 16));	  
            }
            catch (Exception e) {
                ret = sb.toString();
                e.printStackTrace();
            }
        }
        else {
            ret = sb.toString();
        }
        //        if(!StringUtil.isEmpty(ret)){
        //            ret = URLDecoder.decode(ret, enc);
        //        }
        LogUtil.i(TAG, "网络返回数据：" + ret);
        return ret;
    }
    
    /**
     * 获取url中的文件大小<BR>
     * [功能详细描述]
     * @param [url]
     * @return [len]
     */
    public static long getFileSizeAtURL(URL url) {
        int filesize = -1;
        try {
            HttpURLConnection http = (HttpURLConnection) url.openConnection();
            filesize = http.getContentLength();
            http.disconnect();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return filesize;
    }
}
