/*
 * 文件名: UnionSdk.java
 * 版    权：  Copyright Changwan Tech. Co. Ltd. All Rights Reserved.
 * 描    述: [该类的简要描述]
 * 创建人: turlet@163.com
 * 创建时间:2013-5-6
 * 
 * 修改人：
 * 修改时间:
 * 修改内容：[修改内容]
 */
package cn.ewan.pushsdk.open;

import android.content.Context;
import android.os.Handler;
import android.os.Looper;
import android.widget.Toast;
import cn.ewan.pushsdk.internal.EwanPushCliendApi;
import cn.ewan.pushsdk.listener.Callback;
import cn.ewan.pushsdk.logic.CacheData;
import cn.ewan.pushsdk.logic.EwanPushService;
import cn.ewan.pushsdk.model.Response;
import cn.ewan.pushsdk.model.ResponseInit;
import cn.ewan.pushsdk.util.Common;
import cn.ewan.pushsdk.util.LogUtil;
import cn.ewan.pushsdk.util.StringUtil;

/**
 * [联运SDK对外开放接口]<BR>
 * [功能详细描述]
 * @author turlet@163.com
 * @version [Client V100R001C03, 2013-5-6] 
 */
public class EwanPushClient {
	
    /**
     * 单例
     */
    private static EwanPushClient instance;
    
    private boolean initSuc = false;
    
	private Handler mMainThreadHandler = new Handler(Looper.getMainLooper());
	
	private String appId = "";
	
	private EwanPushDataCallback dataCallback;
	
    
	private EwanPushClient() 
	{
	}

    /**
     * 获取单例实体<BR>
     * [功能详细描述]
     * @return instance
     */
    public synchronized static EwanPushClient getInstance() {
        if (null == instance) {
        	LogUtil.i("EwanPushClient", "instance create==============");
            instance = new EwanPushClient();
        }
				
        return instance;
    }
        
    /**
     * [初始化接口]<BR>
     * @param [ctx] 上下文
     * @param [info] 初始化信息
     * @param [listener] 回调监听
     */
    public void EwanPushInit(final Context ctx, 
    		InitInfo info,
            final EwanPushCallback callback,
            final EwanPushDataCallback dataCallBack) {
		Common.debugModel = info.getDebugMode();
		Common.setInitProtocol(true);
		CacheData.getInstance().setProtocolKey(ctx,info.getSignKey());
		appId = info.getAppId();
		setDataCallback(dataCallBack);
        switch (Common.debugModel) {
        case 0:
            Common.HTTP_EWANPUSH_URL = Common.HTTP_TEST_URL;
            break;
        case 1:
            Common.HTTP_EWANPUSH_URL = Common.HTTP_NORMAL_URL;
            break;
        }
        StringBuilder sb = new StringBuilder();
        sb.append(Common.HTTP_EWANPUSH_URL)
           .append("address")
           .append("/")
           .append(appId);
        
        EwanPushService.init(ctx, 
        		info.getAppId(), 
        		info.getSignKey(), 
        		sb.toString(), 
        		info.getUserId(), 
        		new Callback()
				{

					@Override
					public void onSuccess(Response response)
					{
						ResponseInit initInfo = (ResponseInit) response;
						LogUtil.i("EwanPushClient", "uri ==="+initInfo.getAdress());
						LogUtil.i("EwanPushClient", "cliendId ==="+initInfo.getCliendId());
						LogUtil.i("EwanPushClient", "userId ==="+initInfo.getCliendId());
//						WriteLog.e("EwanPushInit  Success", "uri ==="+initInfo.getAdress());
//						WriteLog.e("EwanPushInit  Success", "cliendId ==="+initInfo.getCliendId());
						EwanPushCliendApi.getInstance(ctx).connect(initInfo.getAdress(), 
								initInfo.getCliendId(), 
								new EwanPushCallback()
								{
									
									@Override
									public void onSuccess()
									{
										runOnMainThread(new Runnable()
										{
											
											@Override
											public void run()
											{
/*												setInitSuc(true);
												String temp = CacheData.getInstance().getTopics(ctx);
												if(!StringUtil.isEmpty(temp)){
													String array[] = temp.split(",");
													if(array != null && array.length > 0){														
														EwanPushSubscribe(ctx, array, new EwanPushCallback()
														{
															
															@Override
															public void onSuccess()
															{
																LogUtil.i("", "reconect subscribe success");
															}
															
															@Override
															public void onFail(int errorCode, String msg)
															{
																LogUtil.i("", "reconect subscribe fail");
															}
														});
													}
													for(int i=0; i < array.length; i ++){
														if(!StringUtil.isEmpty(array[i])){
//															WriteLog.e("EwanPushSubscribe", "topicId ==="+array[i]);
															EwanPushSubscribe(ctx, array[i], new EwanPushCallback()
															{
																
																@Override
																public void onSuccess()
																{
																	LogUtil.i("", "reconect subscribe success");
//																	WriteLog.e("EwanPushSubscribe", "subscribe success==================");
																}
																
																@Override
																public void onFail(int errorCode, String msg)
																{
																	LogUtil.i("", "reconect subscribe fail");
//																	WriteLog.e("EwanPushConnect", "conect fail==================");
																}
															});
														}
													}
												}else{		*/
													setInitSuc(true);

													if(callback != null){
														callback.onSuccess();
													}
												}
//											}
										});
									}
									
									@Override
									public void onFail(int errorCode, String msg)
									{
										// TODO 自动生成的方法存根
										LogUtil.i("", "init fail msg ===="+msg);
										setInitSuc(false);
										if(callback != null){
											callback.onFail(errorCode, msg);
										}
									}
								});
					}
					
					@Override
					public void onFail(int errorCode, String msg)
					{
						setInitSuc(false);
//						WriteLog.e("EwanPushInit  fail", msg);
						if(callback != null){
							callback.onFail(errorCode, msg);
						}
					}
				});
    }

    
    /**
     * [账号绑定]<BR>
     * @param [ctx] 上下文
     * @param [userId] 用户账号id
     * @param [listener] 回调监听
     */
    public void EwanPushBindAccout(Context ctx,
    		String userId,final EwanPushCallback callback) {
    	
    	if(!isInitSuc()){
    		ewanToast(ctx,"尚未初始化，请重新初始化~");
    		return;
    	}
    	Common.setInitProtocol(false);
        StringBuilder sb = new StringBuilder();
        sb.append(Common.HTTP_EWANPUSH_URL)
	        .append("bind")           
	        .append("/")
	        .append(appId);
        
        EwanPushService.bindAccount(ctx, appId, 
        		userId, 
        		sb.toString(), 
        		new Callback()
				{
					
					@Override
					public void onSuccess(Response response)
					{
//						WriteLog.e("EwanPushBindAccout","bind success=============");
						if(callback != null){
							callback.onSuccess();
						}
					}
					
					@Override
					public void onFail(int errorCode, String msg)
					{
//						WriteLog.e("EwanPushBindAccout","bind fail=============");
						if(callback != null){
							callback.onFail(errorCode, msg);
						}
					}
				});
    }
    
    
    /**
     * [订阅主题]<BR>
     * @param [ctx] 上下文
     * @param [topics] 订阅号
     * @param [listener] 回调监听
     */
    public void EwanPushSubscribe(Context ctx, String topics, EwanPushCallback callback) {
    	LogUtil.i("EwanPushClient", "EwanPushSubscribe  ==============");
    	if(!isInitSuc()){
    		ewanToast(ctx,"尚未初始化，请重新初始化~");
    		return;
    	}
    	
    	if(StringUtil.isEmpty(topics)){
    		ewanToast(ctx,"订阅主题为空~~~~~");
    		return;
    	}
    	EwanPushCliendApi.getInstance(ctx).subscribe(topics, callback);
    }
    
    /**
     * [批量订阅主题]<BR>
     * @param [ctx] 上下文
     * @param [topics] 订阅号数组
     * @param [listener] 回调监听
     */
    public void EwanPushSubscribe(Context ctx, String[] topics, EwanPushCallback callback) {
    	LogUtil.i("EwanPushClient", "EwanPushSubscribe  ==============");
    	if(!isInitSuc()){
    		ewanToast(ctx,"尚未初始化，请重新初始化~");
    		return;
    	}
    	
    	if(topics == null || topics.length <= 0){
    		ewanToast(ctx,"订阅主题为空~~~~~");
    		return;
    	}
    	EwanPushCliendApi.getInstance(ctx).subscribe(topics, callback);
    }
    
    /**
     * [取消主题订阅]<BR>
     * @param [ctx] 上下文
     * @param [topics] 订阅号
     * @param [listener] 回调监听
     */
    public void EwanPushUnSubscribe(Context ctx, String topics, EwanPushCallback callback) {
    	if(!isInitSuc()){
    		ewanToast(ctx,"尚未初始化，请重新初始化~");
    		return;
    	}
    	
    	if(StringUtil.isEmpty(topics)){
    		ewanToast(ctx,"取消订阅主题为空~~~~~");
    		return;
    	}
//    	WriteLog.e("EwanPushUnSubscribe", "EwanPushUnSubscribe unsubscribe topicId===="+topics);
    	EwanPushCliendApi.getInstance(ctx).unSubscribe(topics, callback);
    }
    
    
    /**
     * [取消主题订阅]<BR>
     * @param [ctx] 上下文
     * @param [topics] 订阅号数组
     * @param [listener] 回调监听
     */
    public void EwanPushUnSubscribe(Context ctx, String[] topics, EwanPushCallback callback) {
    	if(!isInitSuc()){
    		ewanToast(ctx,"尚未初始化，请重新初始化~");
    		return;
    	}
    	
    	if(topics == null || topics.length <= 0){
    		ewanToast(ctx,"取消订阅主题为空~~~~~");
    		return;
    	}
//    	WriteLog.e("EwanPushUnSubscribe", "EwanPushUnSubscribe unsubscribe topicId===="+topics);
    	EwanPushCliendApi.getInstance(ctx).unSubscribe(topics, callback);
    }
   
	private void runOnMainThread(Runnable runnable) {
		mMainThreadHandler.post(runnable);
	}

	
    /**
     * [发布订阅主题消息]<BR>
     * @param [ctx] 上下文
     * @param [topics] 订阅号
     * @param [message] 消息
     * @param [listener] 回调监听
     */
    public void EwanPushPublish(Context ctx, String topics, String message, EwanPushCallback callback) {
    	if(!isInitSuc()){
    		ewanToast(ctx,"尚未初始化，请重新初始化~");
    		return;
    	}
    	
    	if(StringUtil.isEmpty(topics) ){
    		ewanToast(ctx,"发送订阅主题为空~~~~~");
    		return;
    	}
    	
    	if(StringUtil.isEmpty(message) ){
    		ewanToast(ctx,"发送消息为空~~~~~");
    		return;
    	}
    	
    	EwanPushCliendApi.getInstance(ctx).publish(topics, message, callback);
    }
    
    /**
     * [断开推送SDK连接]<BR>
     * @param [ctx] 上下文
     */
    public void EwanPushDisConnect(Context ctx){
    	setInitSuc(false);
//    	WriteLog.e("EwanPushDisConnect", "disConnect ==================");
    	EwanPushCliendApi.getInstance(ctx).disConnect();
    }

    
	private void ewanToast(final Context ctx,final String msg){
		runOnMainThread(new Runnable()
		{
			
			@Override
			public void run()
			{
	            Toast.makeText(ctx,
	            		msg,
	                    Toast.LENGTH_LONG).show();
			}
		});
	}
	
	/**
	 * @return initSuc
	 */
	public boolean isInitSuc()
	{
		LogUtil.i("isInitSuc", "initSuc ==="+initSuc);
//		WriteLog.e("isInitSuc", "initSuc ==="+initSuc);
		return initSuc;
	}

	/**
	 * @param initSuc 要设置的 initSuc
	 */
	public void setInitSuc(boolean initSuc)
	{
		LogUtil.i("setInitSuc", "initSuc ==="+initSuc);
		this.initSuc = initSuc;
	}

	/**
	 * @return dataCallback
	 */
	public EwanPushDataCallback getDataCallback()
	{
		return dataCallback;
	}

	/**
	 * @param dataCallback 要设置的 dataCallback
	 */
	public void setDataCallback(EwanPushDataCallback dataCallback)
	{
		this.dataCallback = dataCallback;
	}

	
}
