/*
 * 文件名: DeviceInfo.java
 * 版    权：  Copyright iAround Tech. Co. Ltd. All Rights Reserved.
 * 描    述: [该类的简要描述]
 * 创建人: turlet@163.com
 * 创建时间:2012-11-9
 * 
 * 修改人：
 * 修改时间:
 * 修改内容：[修改内容]
 */
package cn.ewan.pushsdk.util;

import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.util.Enumeration;
import java.util.Locale;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.database.Cursor;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.Uri;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.os.Build;
import android.provider.Settings.Secure;
import android.support.v4.app.ActivityCompat;
import android.telephony.TelephonyManager;
import android.text.TextUtils;
import android.util.DisplayMetrics;

/**
 * 设置信息<BR>
 * [功能详细描述]
 * @author turlet@163.com
 * @version [RCS Client V100R001C03, 2012-11-9] 
 */
public class DeviceInfo {
    
    /**
     * 获取运行平台<BR>
     * [功能详细描述]
     * @return
     */
    public static String getPlatType() {
        //平台标识（1:android;2:iphone）
        return "1";
    }
    
    /**
     * 获取本地语言<BR>
     * 语言 // 1:英文 ，2：中文简体，3：中文繁体（其他语言默认为1：英文）
     * @return
     */
    public static String getLocalLanguage() {
        String language = Locale.getDefault().getCountry();
        if (StringUtil.isEmpty(language)) {
            return "1";
        }
        if (language.equalsIgnoreCase("CN")) {
            return "2";
        }
        else if (language.equalsIgnoreCase("TW")) {
            return "3";
        }
        return "1";
    }
    
    /**
     * 设备唯一码<BR>
     * [功能详细描述]
     * @param context
     * @return
     */
    public static String getDeviceID(Context context) {
        String deviceID = Secure.getString(context.getContentResolver(),
                Secure.ANDROID_ID);
        return deviceID;
    }
    
    /**
     * 设备IMSI<BR>
     * [功能详细描述]
     * @param context
     * @return
     */
    public static String getIMSI(Context context) {
    	String imsi = null;
        if (Build.VERSION.SDK_INT < 23 ||
                ActivityCompat.checkSelfPermission(context, Manifest.permission.READ_PHONE_STATE) == PackageManager.PERMISSION_GRANTED) {
            try {
                TelephonyManager tm = (TelephonyManager) context
                        .getSystemService(Context.TELEPHONY_SERVICE);
                imsi = tm.getSubscriberId();
            } catch (Exception e) {
                imsi = null;
            }
        }
        if (StringUtil.isEmpty(imsi)) {
            imsi = "0";
        }
        return imsi;
    }
    
    /**
     * 设备IMEI<BR>
     * [功能详细描述]
     * @param context
     * @return
     */
    public static String getIMEI(Context context) {
        String imei = null;
        if (Build.VERSION.SDK_INT < 23 ||
                ActivityCompat.checkSelfPermission(context, Manifest.permission.READ_PHONE_STATE) == PackageManager.PERMISSION_GRANTED) {
            try {
                TelephonyManager tm = (TelephonyManager) context
                        .getSystemService(Context.TELEPHONY_SERVICE);
                imei = tm.getDeviceId();
            } catch (Exception e) {
                imei = null;
            }
        }
        if (StringUtil.isEmpty(imei)) {
            imei = "0";
        }
        return imei;
    }
    
    /**
     * MAC地址 2c:d9:89:66:a5<BR>
     * [功能详细描述]
     * @param context
     * @return
     */
    public static String getMacAddress(Context context) {
        WifiManager wifi = (WifiManager) context.getSystemService(Context.WIFI_SERVICE);
        WifiInfo info = wifi.getConnectionInfo();
        String mac = info.getMacAddress();
        if (StringUtil.isEmpty(mac)) {
            mac = "0:0:0:0:0:0";
        }
        return mac;
    }
    
    /**
     * sdk版本ID 8<BR>
     * [功能详细描述]
     * @return
     */
    public static String getSDKVersion() {
        return String.valueOf(Build.VERSION.SDK_INT);
    }
    
    /**
     * 手机型号 LT22i<BR>
     * [功能详细描述]
     * @return
     */
    public static String getModel() {
        return getMake() + " " + Build.MODEL;
    }
    
    /**
     * 手机生产商<BR>
     * samsung
     * @return
     */
    public static String getMake() {
        return Build.MANUFACTURER;
    }
    
    /**
     * 系统版本号 2.2<BR>
     * [功能详细描述]
     * @return
     */
    public static String getSysVersion() {
        return Build.VERSION.RELEASE;
    }
    
    /**
     * 分辨率 480*800<BR>
     * [功能详细描述]
     * @return
     */
    public static String getResolution(Context context) {
        if (context instanceof Activity) {
            DisplayMetrics displaysMetrics = new DisplayMetrics();
            ((Activity) context).getWindowManager()
                    .getDefaultDisplay()
                    .getMetrics(displaysMetrics);
            
            //得到屏幕宽高
            return displaysMetrics.widthPixels + "*"
                    + displaysMetrics.heightPixels;
        }
        return "0*0";
    }
    
    public static int getWidth(Context context) {
        if (context instanceof Activity) {
            DisplayMetrics displaysMetrics = new DisplayMetrics();
            ((Activity) context).getWindowManager()
                    .getDefaultDisplay()
                    .getMetrics(displaysMetrics);
            
            //得到屏幕宽
            return displaysMetrics.widthPixels;
        }
        return 0;
    }
    
    public static int getHeight(Context context) {
        if (context instanceof Activity) {
            DisplayMetrics displaysMetrics = new DisplayMetrics();
            ((Activity) context).getWindowManager()
                    .getDefaultDisplay()
                    .getMetrics(displaysMetrics);
            //得到屏幕宽
            return displaysMetrics.heightPixels;
        }
        return 0;
    }
    
    /**
     * [获取IP]<BR>
     * [功能详细描述]
     * @return
     */
    public static String getIp() {
        try {
            for (Enumeration<NetworkInterface> en = NetworkInterface.getNetworkInterfaces(); en.hasMoreElements();) {
                NetworkInterface intf = (NetworkInterface) en.nextElement();
                for (Enumeration<InetAddress> enumIpAddr = intf.getInetAddresses(); enumIpAddr.hasMoreElements();) {
                    InetAddress inetAddress = (InetAddress) enumIpAddr.nextElement();
                    if (!inetAddress.isLoopbackAddress())
                        return inetAddress.getHostAddress().toString();
                }
            }
        }
        catch (SocketException ex) {
            LogUtil.d("TAG", ex.toString());
        }
        return "";
    }
    
    /**versionCode = 1234
     * 当前应用版本<BR>
     * [功能详细描述]
     * @param context
     * @return
     */
    public static int getAppVersion(Context context) {
        try {
            PackageInfo info = context.getPackageManager()
                    .getPackageInfo(context.getPackageName(), 0);
            return info.versionCode;
        }
        catch (NameNotFoundException e) {
            e.printStackTrace();
        }
        return 0;
    }
    
    /**V1.0.0.1
     *  当前应用版本<BR>
     * [功能详细描述]
     * @param context
     * @return
     */
    public static String getAppVersionName(Context context) {
        try {
            PackageInfo info = context.getPackageManager()
                    .getPackageInfo(context.getPackageName(), 0);
            return info.versionName;
        }
        catch (NameNotFoundException e) {
            e.printStackTrace();
        }
        return "1.0.0";
    }
    
    /**
     * 获取Network具体类型
     * @param mContext
     * @return wifi
     */
    public static String getCarriers(Context context) {
        Uri PREFERRED_APN_URI = Uri.parse("content://telephony/carriers/preferapn");
        try {
            final ConnectivityManager connectivityManager = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
            final NetworkInfo networkInfo = connectivityManager.getActiveNetworkInfo();
            if (networkInfo == null || !networkInfo.isAvailable()) {
                return "other";
            }
            else {
                int netType = networkInfo.getType();
                if (netType == ConnectivityManager.TYPE_WIFI) {
                    return "WiFi";
                }
                else if (netType == ConnectivityManager.TYPE_MOBILE) {
                    // 电信wap
                    final Cursor c = context.getContentResolver()
                            .query(PREFERRED_APN_URI, null, null, null, null);
                    try {
                        if (c != null) {
                            c.moveToFirst();
                            final String user = c.getString(c.getColumnIndex("user"));
                            if (!TextUtils.isEmpty(user)) {
                                return user;
                            }
                        }
                        String netMode = networkInfo.getExtraInfo();
                        if (netMode != null) {
                            netMode = netMode.toLowerCase();
                            return netMode;
                        }
                    }
                    catch (Exception e) {
                        e.printStackTrace();
                    }
                    finally {
                        if (c != null) {
                            c.close();
                        }
                    }
                }
            }
        }
        catch (Exception ex) {
            return "other";
        }
        return "other";
    }
    
    
    /**
     * 判断是否为3G或3G以上网络
     * 
     * @param mContext
     * @return 
     */  
    private static boolean isFastMobileNetwork(Context context) {  
    	TelephonyManager telephonyManager = (TelephonyManager)context.getSystemService(Context.TELEPHONY_SERVICE);  
    	switch (telephonyManager.getNetworkType()) {  
    	       case TelephonyManager.NETWORK_TYPE_1xRTT:  
    	           return false; // ~ 50-100 kbps  
    	       case TelephonyManager.NETWORK_TYPE_CDMA:  
    	           return false; // ~ 14-64 kbps  
    	       case TelephonyManager.NETWORK_TYPE_EDGE:  
    	           return false; // ~ 50-100 kbps  
    	       case TelephonyManager.NETWORK_TYPE_EVDO_0:  
    	           return true; // ~ 400-1000 kbps  
    	       case TelephonyManager.NETWORK_TYPE_EVDO_A:  
    	           return true; // ~ 600-1400 kbps  
    	       case TelephonyManager.NETWORK_TYPE_GPRS:  
    	           return false; // ~ 100 kbps  
    	       case TelephonyManager.NETWORK_TYPE_HSDPA:  
    	           return true; // ~ 2-14 Mbps  
    	       case TelephonyManager.NETWORK_TYPE_HSPA:  
    	           return true; // ~ 700-1700 kbps  
    	       case TelephonyManager.NETWORK_TYPE_HSUPA:  
    	           return true; // ~ 1-23 Mbps  
    	       case TelephonyManager.NETWORK_TYPE_UMTS:  
    	           return true; // ~ 400-7000 kbps  
    	       case TelephonyManager.NETWORK_TYPE_EHRPD:  
    	           return true; // ~ 1-2 Mbps  
    	       case TelephonyManager.NETWORK_TYPE_EVDO_B:  
    	           return true; // ~ 5 Mbps  
    	       case TelephonyManager.NETWORK_TYPE_HSPAP:  
    	           return true; // ~ 10-20 Mbps  
    	       case TelephonyManager.NETWORK_TYPE_IDEN:  
    	           return false; // ~25 kbps  
    	       case TelephonyManager.NETWORK_TYPE_LTE:  
    	           return true; // ~ 10+ Mbps  
    	       case TelephonyManager.NETWORK_TYPE_UNKNOWN:  
    	           return false;  
    	       default:  
    	           return false;  
    	    }  
    	}  
    
    /** 
     * 获取网络状态，wifi,wap,2g,3g. 
     * 
     * @param context 上下文 
     * @return int 网络状态 {@link #NETWORKTYPE_2G},{@link #NETWORKTYPE_3G},          *{@link #NETWORKTYPE_INVALID},{@link #NETWORKTYPE_WAP}* <p>{@link #NETWORKTYPE_WIFI} 
     */  
  
    public static String getNetWorkType(Context context) {  
  
        ConnectivityManager manager = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);  
        NetworkInfo networkInfo = manager.getActiveNetworkInfo();  
        String mNetWorkType = "";
  
        if (networkInfo != null && networkInfo.isConnected()) {  
            String type = networkInfo.getTypeName();  
  
            if (type.equalsIgnoreCase("WIFI")) {  
                mNetWorkType = "WIFI";  
            } else if (type.equalsIgnoreCase("MOBILE")) {  
                String proxyHost = android.net.Proxy.getDefaultHost();  
  
                mNetWorkType = TextUtils.isEmpty(proxyHost)  
                        ? (isFastMobileNetwork(context) ? "3G" : "2G")  
                        : "Wap";  
            }  
        } else {  
            mNetWorkType = "";  
        }  
  
        return mNetWorkType;  
    }   
}
