package cn.ewan.pushsdk.util;


import android.content.Context;
import android.content.SharedPreferences;

/**
 * 这是一个保存共享的数据工具类<BR>
 * [功能详细描述]
 * @author 杨兴朗
 * @version [RCS Client V100R001C03, 2012-11-7] 
 */
public class SPUtil {
    /**
     * SharedPreferences文件名
     */
    private static final String PREFERENCE_NAME = "ewanpushsdk";
    
    public static final String usertoken = "usertoken";
    public static final String uri = "uriAdress";
    public static final String key = "key";
    public static final String protocolkey = "protocolkey";
    public static final String topics = "topics";
    
    private SharedPreferences mSharedPreferences;
    
    private static SPUtil instance;
    
    private SPUtil(Context context) {
        mSharedPreferences = context.getSharedPreferences(PREFERENCE_NAME,
                Context.MODE_PRIVATE);
    }
    
    public static SPUtil getInstance(Context context) {
        if (instance == null) {
            synchronized (SPUtil.class) {
                if (instance == null) {
                    instance = new SPUtil(context);
                }
            }
        }
        return instance;
    }
    
    /**
     * 保存字符串
     * @param key
     * @param value
     */
    public void saveString(String key, String value) {
        if (!StringUtil.isEmpty(value)) {
            value = String.valueOf(Base64.encode(value.getBytes()));
        }
        mSharedPreferences.edit().putString(key, value).commit();
    }
    
    /**
     * 获取字符串
     * @param key
     * @param defValue
     * @return bool
     */
    public String getString(String key, String... defValue) {
        if (defValue.length > 0) {
            String value = mSharedPreferences.getString(key, defValue[0]);
            if (StringUtil.isEmpty(value) || value.equals(defValue[0])) {//如果为不空或者等于默认值则返回
                return defValue[0];
            }
            try {
                return new String(Base64.decode(value.toCharArray()),
                        Common.ENC_UTF_8);
            }
            catch (Exception e) {
                return defValue[0];
            }
        }
        else {
            String value = mSharedPreferences.getString(key, "");
            if (StringUtil.isEmpty(value)) {
                return "";
            }
            try {
                return new String(Base64.decode(value.toCharArray()),
                        Common.ENC_UTF_8);
            }
            catch (Exception e) {
                return value;
            }
        }
    }
    
    /**
     * 保存布尔值
     * @param key
     * @param value
     */
    public void saveBoolean(String key, Boolean value) {
        mSharedPreferences.edit().putBoolean(key, value).commit();
    }
    
    /**
     * 获取布尔值
     * @param key
     * @param defValue
     * @return bool
     */
    public Boolean getBoolean(String key, Boolean... defValue) {
        if (defValue.length > 0) {
            return mSharedPreferences.getBoolean(key, defValue[0]);
        }
        else {
            return mSharedPreferences.getBoolean(key, false);
        }
    }
    
    /**
     * 保存整形
     * @param key
     * @param value
     */
    public void saveInteger(String key, Integer value) {
        mSharedPreferences.edit().putInt(key, value).commit();
    }
    
    /**
     * 获取整形
     * @param key
     * @param defValue
     * @return int
     */
    public Integer getInteger(String key, Integer... defValue) {
        if (defValue.length > 0) {
            return mSharedPreferences.getInt(key, defValue[0]);
        }
        else {
            return mSharedPreferences.getInt(key, 0);
        }
    }
    
    /**
     * 保存长整形
     * @param key
     * @param value
     */
    public void saveLong(String key, Long value) {
        mSharedPreferences.edit().putLong(key, value).commit();
        
    }
    
    /**
     * 获取长整形
     * @param key
     * @param defValue
     * @return long
     */
    public Long getLong(String key, Long... defValue) {
        if (defValue.length > 0) {
            return mSharedPreferences.getLong(key, defValue[0]);
        }
        else {
            return mSharedPreferences.getLong(key, 0);
        }
    }
    
    /**
     * 配置中是否存在key<BR>
     * [功能详细描述]
     * @param key
     * @return
     */
    public boolean hasKey(String key) {
        return mSharedPreferences.contains(key);
    }
}
